@@ -113,9 +113,9 @@ def dialog( toplevel_window, title, question, description ):
 def get_file_from_filechooser( toplevel_window, save_file=False, save_to=None):
     if util.platform == util.MAEMO:
         if save_file:
-            args = ( toplevel_window, gtk.FILE_CHOOSER_ACTION_SAVE )
+            args = ( toplevel_window, gtk.FILE_CHOOSER_ACTION_SAVE, hildon.FileSystemModel() )
         else:
-            args = ( toplevel_window, gtk.FILE_CHOOSER_ACTION_OPEN )
+            args = ( toplevel_window, gtk.FILE_CHOOSER_ACTION_OPEN, hildon.FileSystemModel())
 
         dlg = hildon.FileChooserDialog( *args )
     else:
@@ -149,9 +149,11 @@ def get_file_from_filechooser( toplevel_window, save_file=False, save_to=None):
     dlg.destroy()
     return filename
 
-class BookmarksWindow(gtk.Window):
+class BookmarksWindow(hildon.StackableWindow):
     def __init__(self, main_window):
-        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
+	hildon.StackableWindow.__init__(self)
+	self.set_transient_for(main_window.main_window)
+	self.connect('destroy', self.on_destroy)
         self.__log = logging.getLogger('panucci.panucci.BookmarksWindow')
         self.main = main_window
         self.main.bookmarks_window_open = True
@@ -204,24 +206,23 @@ class BookmarksWindow(gtk.Window):
         sw.set_shadow_type(gtk.SHADOW_IN)
         sw.add(self.treeview)
         self.vbox.add(sw)
-        self.hbox = gtk.HButtonBox()
+
+        app_menu = hildon.AppMenu()
         self.add_button = gtk.Button(gtk.STOCK_ADD)
         self.add_button.set_use_stock(True)
         self.add_button.connect('clicked', self.add_bookmark)
-        self.hbox.pack_start(self.add_button)
+	app_menu.append(self.add_button)
         self.remove_button = gtk.Button(gtk.STOCK_REMOVE)
         self.remove_button.set_use_stock(True)
         self.remove_button.connect('clicked', self.remove_bookmark)
-        self.hbox.pack_start(self.remove_button)
+	app_menu.append(self.remove_button)
         self.jump_button = gtk.Button(gtk.STOCK_JUMP_TO)
         self.jump_button.set_use_stock(True)
         self.jump_button.connect('clicked', self.jump_bookmark)
-        self.hbox.pack_start(self.jump_button)
-        self.close_button = gtk.Button(gtk.STOCK_CLOSE)
-        self.close_button.set_use_stock(True)
-        self.close_button.connect('clicked', self.close)
-        self.hbox.pack_start(self.close_button)
-        self.vbox.pack_start(self.hbox, False, True)
+	app_menu.append(self.jump_button)
+	app_menu.show_all()
+	self.set_app_menu(app_menu)
+
         self.add(self.vbox)
         self.show_all()
 
@@ -283,10 +284,9 @@ class BookmarksWindow(gtk.Window):
         self.treeview.set_model(self.model)
         self.treeview.expand_all()
 
-    def close(self, w):
+    def on_destroy(self, w):
         player.playlist.update_bookmarks()
         self.main.bookmarks_window_open = False
-        self.destroy()
 
     def label_edited(self, cellrenderer, path, new_text):
         iter = self.model.get_iter(path)
@@ -347,8 +347,10 @@ class BookmarksWindow(gtk.Window):
             # FIXME: The player/playlist should be able to take care of this
             if not player.playing:
                 player.play()
+	self.destroy()
 
 class GTK_Main(object):
+    LANDSCAPE, PORTRAIT = range(2)
 
     def __init__(self, filename=None):
         self.__log = logging.getLogger('panucci.panucci.GTK_Main')
@@ -357,11 +359,13 @@ class GTK_Main(object):
 
         self.fullscreen = False
 
+        self.orientation = -1
+
         self.recent_files = []
         self.progress_timer_id = None
         self.volume_timer_id = None
-        self.make_main_window()
         self.has_coverart = False
+        self.make_main_window()
         self.bookmarks_window_open = False
         self.set_volume(settings.volume)
 
@@ -378,12 +382,115 @@ class GTK_Main(object):
         player.playlist.register( 'file_queued', self.on_file_queued )
         player.init(filepath=filename)
 
+    def on_size_allocate(self, widget, allocation):
+        ratio = float(allocation.width)/float(allocation.height)
+        if ratio > 1.:
+            new_orientation = self.LANDSCAPE
+        else:
+            new_orientation = self.PORTRAIT
+        if self.orientation != new_orientation:
+            self.orientation = new_orientation
+            self.orientation_changed()
+
+    def orientation_changed(self):
+        print 'orientation has changed to', self.orientation
+
+        def gtk_box_reparent(b, w, expand=False, fill=True):
+            box = gtk.HBox()
+            b.pack_start(box, expand, fill)
+            if w.get_parent():
+                w.reparent(box)
+            else:
+                box.add(w)
+
+        if self.orientation == self.LANDSCAPE:
+            main_vbox = gtk.VBox()
+            main_vbox.set_spacing(6)
+            self.main_hbox.pack_start(main_vbox, True, True)
+
+            # Horizontal layout: cover art and metadata widget
+            metadata_hbox = gtk.HBox()
+            metadata_hbox.set_spacing(6)
+
+            self.cover_art_event_box.reparent(metadata_hbox)
+            self.metadata_widget.reparent(metadata_hbox)
+            gtk_box_reparent(metadata_hbox, self.cover_art_event_box, False, False)
+            gtk_box_reparent(metadata_hbox, self.metadata_widget, True, True)
+            main_vbox.pack_start(metadata_hbox, True, False)
+
+            # Horizontal layout: add progress widget
+            gtk_box_reparent(main_vbox, self.progress_widget, False, False)
+
+            # Horizontal layout: all buttons in the buttonbox
+            buttonbox = gtk.HBox()
+            gtk_box_reparent(buttonbox, self.rrewind_button, True, True)
+            gtk_box_reparent(buttonbox, self.rewind_button, True, True)
+            gtk_box_reparent(buttonbox, self.play_pause_button, True, True)
+            gtk_box_reparent(buttonbox, self.forward_button, True, True)
+            gtk_box_reparent(buttonbox, self.fforward_button, True, True)
+            gtk_box_reparent(buttonbox, self.bookmarks_button, True, True)
+            gtk_box_reparent(buttonbox, self.volume_button, True, True)
+            main_vbox.pack_start(buttonbox, False, False)
+
+            for child in self.main_hbox.get_children():
+                if child != main_vbox and child != self.volume:
+                    self.main_hbox.remove(child)
+        else:
+            main_vbox = gtk.VBox()
+            main_vbox.set_spacing(6)
+            self.main_hbox.pack_start(main_vbox, True, True)
+
+            # Horizontal layout: cover art and metadata widget
+            gtk_box_reparent(main_vbox, self.cover_art_event_box, False, False)
+            gtk_box_reparent(main_vbox, self.metadata_widget, True, True)
+
+            # Horizontal layout: add progress widget
+            gtk_box_reparent(main_vbox, self.progress_widget, False, False)
+
+            # Horizontal layout: all buttons in the buttonbox
+            buttontbl = gtk.Table(2, 4, True)
+            buttontbl.set_size_request(-1, 160)
+
+            def gtk_table_reparent(t, w, left, right, top, bottom):
+                b = gtk.HBox()
+                t.attach(b, left, right, top, bottom)
+                if w.get_parent():
+                    w.reparent(b)
+                else:
+                    b.add(w)
+
+            gtk_table_reparent(buttontbl, self.bookmarks_button, 0, 1, 0, 1)
+            gtk_table_reparent(buttontbl, self.play_pause_button, 1, 3, 0, 1)
+            gtk_table_reparent(buttontbl, self.volume_button, 3, 4, 0, 1)
+            gtk_table_reparent(buttontbl, self.rrewind_button, 0, 1, 1, 2)
+            gtk_table_reparent(buttontbl, self.rewind_button, 1, 2, 1, 2)
+            gtk_table_reparent(buttontbl, self.forward_button, 2, 3, 1, 2)
+            gtk_table_reparent(buttontbl, self.fforward_button, 3, 4, 1, 2)
+
+            main_vbox.pack_start(buttontbl, False, False)
+
+            for child in self.main_hbox.get_children():
+                if child != main_vbox and child != self.volume:
+                    self.main_hbox.remove(child)
+
+        for label in (self.title_label, self.artist_label, self.album_label):
+            if self.has_coverart and self.orientation == self.LANDSCAPE:
+                xalign = 0.
+            else:
+                xalign = .5
+            label.set_alignment(xalign, .5)
+
+        if util.platform == util.MAEMO:
+            self.main_hbox.reorder_child(self.volume, -1)
+
+        self.main_window.show_all()
+
     def make_main_window(self):
         import pango
 
         if util.platform == util.MAEMO:
             self.app = hildon.Program()
-            window = hildon.Window()
+            window = hildon.StackableWindow()
             self.app.add_window(window)
         else:
             window = gtk.Window(gtk.WINDOW_TOPLEVEL)
@@ -392,13 +499,15 @@ class GTK_Main(object):
         self.window_icon = util.find_image('panucci.png')
         if self.window_icon is not None:
             window.set_icon_from_file( self.window_icon )
-        window.set_default_size(400, -1)
+        #window.set_default_size(400, -1)
         window.set_border_width(0)
         window.connect("destroy", self.destroy)
+        window.connect('size-allocate', self.on_size_allocate)
         self.main_window = window
 
         if util.platform == util.MAEMO:
-            window.set_menu(self.create_menu())
+            window.set_app_menu(self.create_app_menu())
+            #window.set_menu(self.create_menu())
         else:
             menu_vbox = gtk.VBox()
             menu_vbox.set_spacing(0)
@@ -410,88 +519,58 @@ class GTK_Main(object):
             menu_vbox.pack_start(menu_bar, False, False, 0)
             menu_bar.show()
 
-        main_hbox = gtk.HBox()
-        main_hbox.set_spacing(6)
-        if util.platform == util.MAEMO:
-            window.add(main_hbox)
-        else:
-            menu_vbox.pack_end(main_hbox, True, True, 6)
-
-        main_vbox = gtk.VBox()
-        main_vbox.set_spacing(6)
-        # add a vbox to the main_hbox
-        main_hbox.pack_start(main_vbox, True, True)
-
-        # a hbox to hold the cover art and metadata vbox
-        metadata_hbox = gtk.HBox()
-        metadata_hbox.set_spacing(6)
-        main_vbox.pack_start(metadata_hbox, True, False)
-        
+        # Covert art is in self.cover_art_event_box
         self.cover_art_event_box = gtk.EventBox()
         self.cover_art = gtk.Image()
         self.cover_art_event_box.add(self.cover_art)
         self.cover_art_event_box.connect('button-release-event', self.on_cover_clicked)
-        metadata_hbox.pack_start(self.cover_art_event_box, False, False)
 
-        # vbox to hold metadata
-        metadata_vbox = gtk.VBox()
-        metadata_vbox.set_spacing(8)
-        empty_label = gtk.Label()
-        metadata_vbox.pack_start(empty_label, True, True)
+        # Metadata is always arranged vertically
+        self.metadata_widget = gtk.VBox()
+        self.metadata_widget.set_spacing(8)
+        self.metadata_widget.pack_start(gtk.Label(), True, True)
         self.artist_label = gtk.Label('')
         self.artist_label.set_ellipsize(pango.ELLIPSIZE_END)
-        metadata_vbox.pack_start(self.artist_label, False, False)
+        self.metadata_widget.pack_start(self.artist_label, False, False)
         self.album_label = gtk.Label('')
         self.album_label.set_ellipsize(pango.ELLIPSIZE_END)
-        metadata_vbox.pack_start(self.album_label, False, False)
+        self.metadata_widget.pack_start(self.album_label, False, False)
         self.title_label = gtk.Label('')
         self.title_label.set_line_wrap(True)
-        metadata_vbox.pack_start(self.title_label, False, False)
-        empty_label = gtk.Label()
-        metadata_vbox.pack_start(empty_label, True, True)
-        metadata_hbox.pack_start( metadata_vbox, True, True )
+        self.metadata_widget.pack_start(self.title_label, False, False)
+        self.metadata_widget.pack_start(gtk.Label(), True, True)
 
-        progress_eventbox = gtk.EventBox()
-        progress_eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
-        progress_eventbox.connect('button-press-event', self.on_progressbar_changed)
+        # A single, packable widget for showing progress
+        self.progress_widget = gtk.EventBox()
+        self.progress_widget.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+        self.progress_widget.connect('button-press-event', self.on_progressbar_changed)
         self.progress = gtk.ProgressBar()
         # make the progress bar more "finger-friendly"
         if util.platform == util.MAEMO:
-            self.progress.set_size_request( -1, 50 )
-        progress_eventbox.add(self.progress)
-        main_vbox.pack_start( progress_eventbox, False, False )
+            self.progress.set_size_request(-1, 100)
+        self.progress_widget.add(self.progress)
 
-        # make the button box
-        buttonbox = gtk.HBox()
+        # The buttons
         self.rrewind_button = gtk.Button('')
         image(self.rrewind_button, 'media-skip-backward.png')
         self.rrewind_button.connect('clicked', self.seekbutton_callback, -1*long_seek)
-        buttonbox.add(self.rrewind_button)
         self.rewind_button = gtk.Button('')
         image(self.rewind_button, 'media-seek-backward.png')
         self.rewind_button.connect('clicked', self.seekbutton_callback, -1*short_seek)
-        buttonbox.add(self.rewind_button)
         self.play_pause_button = gtk.Button('')
         image(self.play_pause_button, gtk.STOCK_OPEN, True)
         self.button_handler_id = self.play_pause_button.connect( 
             'clicked', self.open_file_callback )
-        buttonbox.add(self.play_pause_button)
         self.forward_button = gtk.Button('')
         image(self.forward_button, 'media-seek-forward.png')
         self.forward_button.connect('clicked', self.seekbutton_callback, short_seek)
-        buttonbox.add(self.forward_button)
         self.fforward_button = gtk.Button('')
         image(self.fforward_button, 'media-skip-forward.png')
         self.fforward_button.connect('clicked', self.seekbutton_callback, long_seek)
-        buttonbox.add(self.fforward_button)
         self.bookmarks_button = gtk.Button('')
         image(self.bookmarks_button, 'bookmark-new.png')
         self.bookmarks_button.connect('clicked', self.bookmarks_callback)
-        buttonbox.add(self.bookmarks_button)
         self.set_controls_sensitivity(False)
-        main_vbox.pack_start(buttonbox, False, False)
-
-        window.show_all()
 
         if util.platform == util.MAEMO:
             self.volume = hildon.VVolumebar()
@@ -500,7 +579,6 @@ class GTK_Main(object):
             self.volume.connect('mute_toggled', self.mute_toggled)
             window.connect('key-press-event', self.on_key_press)
             window.connect('window-state-event', self.window_state_event)
-            main_hbox.pack_start(self.volume, False, True)
 
             # Add a button to pop out the volume bar
             self.volume_button = gtk.ToggleButton('')
@@ -510,8 +588,6 @@ class GTK_Main(object):
                 'show', lambda x: self.volume_button.set_active(True))
             self.volume.connect(
                 'hide', lambda x: self.volume_button.set_active(False))
-            buttonbox.add(self.volume_button)
-            self.volume_button.show()
 
             # Disable focus for all widgets, so we can use the cursor
             # keys + enter to directly control our media player, which
@@ -525,8 +601,70 @@ class GTK_Main(object):
         else:
             self.volume = gtk.VolumeButton()
             self.volume.connect('value-changed', self.volume_changed_gtk)
-            buttonbox.add(self.volume)
-            self.volume.show()
+            self.volume_button = self.volume
+
+
+        # Generate the basic widget structure
+        self.main_hbox = gtk.HBox()
+        self.main_hbox.set_spacing(6)
+        if util.platform == util.MAEMO:
+            window.add(self.main_hbox)
+        else:
+            menu_vbox.pack_end(self.main_hbox, True, True, 6)
+        self.main_hbox.pack_start(self.volume, False, True)
+
+        # Fill in the widgets in the correct orientation
+        self.orientation = self.LANDSCAPE
+        self.orientation_changed()
+
+    def create_app_menu(self):
+	menu = hildon.AppMenu()
+
+	menu_open = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+	menu_open.set_text(_('Open file'), '')
+        menu_open.connect("clicked", self.open_file_callback)
+        menu.append(menu_open)
+
+	menu_queue = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+	menu_queue.set_text(_('Enqueue file'), '')
+	menu_queue.connect('clicked', self.queue_file_callback)
+        menu.append(menu_queue)
+
+	menu_bookmarks = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+	menu_bookmarks.set_text(_('Bookmarks'), '')
+	menu_bookmarks.connect('clicked', self.bookmarks_callback)
+        menu.append(menu_bookmarks)
+
+	menu_lock_progress = hildon.CheckButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+	menu_lock_progress.set_label('Lock progress bar')
+	menu_lock_progress.set_alignment(.5, .5)
+        menu_lock_progress.connect('toggled', lambda w: 
+            setattr(settings, 'progress_locked', w.get_active()))
+        menu_lock_progress.set_active(self.lock_progress)
+        menu.append(menu_lock_progress)
+
+	menu_about = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+	menu_about.set_text(_('About Panucci'), '')
+        menu_about.connect('clicked', self.show_about, self.main_window)
+        menu.append(menu_about)
+
+	menu.show_all()
+	return menu
+
+        # the recent files menu
+        self.menu_recent = gtk.MenuItem(_('Recent Files'))
+        menu.append(self.menu_recent)
+        self.create_recent_files_menu()
+        
+        menu.append(gtk.SeparatorMenuItem())
+
+        menu.append(gtk.SeparatorMenuItem())
+
+        menu_quit = gtk.ImageMenuItem(gtk.STOCK_QUIT)
+        menu_quit.connect("activate", self.destroy)
+        menu.append(menu_quit)
+
+        return menu
 
     def create_menu(self):
         # the main menu
@@ -908,14 +1046,14 @@ class GTK_Main(object):
             if tags.has_key(tag) and value is not None and value.strip():
                 tags[tag].set_markup('<big>'+value+'</big>')
                 tag_vals[tag] = value
-                tags[tag].set_alignment( 0.5*int(not self.has_coverart), 0.5)
+                tags[tag].set_alignment( 0.5*int(self.orientation == self.PORTRAIT or not self.has_coverart), 0.5)
                 tags[tag].show()
             if tag == 'title':
                 if util.platform == util.MAEMO:
                     self.main_window.set_title(value)
                     # oh man this is hacky :(
                     if self.has_coverart:
-                        tags[tag].set_size_request(420,-1)
+                        #tags[tag].set_size_request(420,-1)
                         if len(value) >= 80: value = value[:80] + '...'
                 else:
                     self.main_window.set_title('Panucci - ' + value)
